#!/usr/bin/python
# coding=utf-8

from flask import *
import time
import mosquitto
from serial import *
import math
import pandas as pd
import wifi

app = Flask(__name__)
clientMosquitto = mosquitto.Mosquitto()

#Variables globales trame JSON
idPatient = 2
nomPatient = "DUPONT"
pnomPatient = "Jean-Luc"
temperature = 0
nFatigue = 0
sechbucc = 0
doulbucc = 0
aphtes = 0
vomi = 0
MG=0
poids=0
taille=0
age=0
horodate=time.strftime('%d/%m/%y %H:%M',time.localtime())
ConfOK = False

#Variables globales MQTT
IPbroker='localhost' #Mettre a localhost pour les tests internes et à 192.168.0.25 pour le serveur médecin
PORTbroker=1883 #Mettre à 1883 pour les tests internes et à 8883 pour le serveur médecin
ConnectLOGIN='patient1'
ConnectMDP='p1agi'
CHEMINtrame='dataPatient/'

#Variables communication série
PORTarduino = '/dev/ttyACM0'
VitesseSerialArduino = 9600
PORTbim="/dev/ttyUSB0"
VitesseSerialBim=115200

#Définition de la communication du BIM
bioimpedancemetre=Serial();
bioimpedancemetre.port = PORTbim #requires driver to be installed (CP210x USB to UART Bridge Virtual COM Port (VCP))
bioimpedancemetre.baudrate = VitesseSerialBim
bioimpedancemetre.bytesize = EIGHTBITS #number of bits per bytes
bioimpedancemetre.parity = PARITY_NONE #set parity check: no parity
bioimpedancemetre.stopbits = STOPBITS_ONE #number of stop bits
bioimpedancemetre.timeout = 1            #non-block read si pb, essayer avec None
bioimpedancemetre.xonxoff = False     #disable software flow control
bioimpedancemetre.rtscts = True     #disable hardware (RTS/CTS) flow control
bioimpedancemetre.dsrdtr = False       #disable hardware (DSR/DTR) flow control
bioimpedancemetre.writeTimeout = 2     #timeout for write

#Variables de l'API-REST
PORTapi=5000

#Definition des events callbacks
def on_connect(mosq,obj,rc):
    if(rc==0):
        print('Connection au Broker OK')
    else :
        print('Exception Connection')
        raise Exception

def on_publish(mosq,obj,mid):
    print("mid : "+str(mid))

#Fontion Wifi
def Search():
    wifilist = []

    cells = wifi.Cell.all('wlan0')

    for cell in cells:
        wifilist.append(cell)

    return wifilist

def FindFromSearchList(ssid):
    wifilist = Search()

    for cell in wifilist:
        if cell.ssid == ssid:
            return cell

    return False


def FindFromSavedList(ssid):
    cell = wifi.Scheme.find('wlan0', ssid)

    if cell:
        return cell

    return False


def Connect(ssid, password=None):
    cell = FindFromSearchList(ssid)

    if cell:
        savedcell = FindFromSavedList(cell.ssid)

        # Already Saved from Setting
        if savedcell:
            savedcell.activate()
            return cell

        # First time to conenct
        else:
            if cell.encrypted:
                if password:
                    scheme = Add(cell, password)

                    try:
                        scheme.activate()

                    # Wrong Password
                    except wifi.exceptions.ConnectionError:
                        Delete(ssid)
                        return False

                    return cell
                else:
                    return False
            else:
                scheme = Add(cell)

                try:
                    scheme.activate()
                except wifi.exceptions.ConnectionError:
                    Delete(ssid)
                    return False

                return True

    return False


def Add(cell, password=None):
    if not cell:
        return False

    scheme = wifi.Scheme.for_cell('wlan0', cell.ssid, cell, password)
    scheme.save()
    return scheme


def Delete(ssid):
    if not ssid:
        return False

    cell = FindFromSavedList(ssid)

    if cell:
        cell.delete()
        return True

    return False

#Assigner les events callbacks
clientMosquitto.on_connect=on_connect
clientMosquitto.on_publish=on_publish

@app.route('/Questionnaire/')
def Questionnaire():
    return render_template('formulaireSimple.html')

@app.route('/rTemp/', methods=['GET'])
def rTemp():
    global temperature
    try:
        arduino=Serial(port=PORTarduino,baudrate=VitesseSerialArduino, timeout=None, writeTimeout=1, dsrdtr=True)
        time.sleep(2)
        arduino.write('temperature')
        time.sleep(1)
        arduino.flushInput()
        arduino.flushOutput()
        temperatureStr=arduino.read(5)
        temperature=float(temperatureStr)
        ret_data = {"temperature": temperature, "etat":'OK'}
        return jsonify(ret_data)
    except:
        ret_data={"etat":'NOK', "temperature":None}
        return jsonify(ret_data)

@app.route('/Send/',methods=['GET'])
def Send():
    global nFatigue,sechbucc, doulbucc, aphtes, vomi, horodate, age, taille, poids, idPatient,nomPatient,pnomPatient,temperature,MG
    nFatigue=int(request.args.get('nfat'))
    if(request.args.get('sechbucc')=='on'):
        sechbucc=1
    else:
        sechbucc=0
    if(request.args.get('doulbucc')=='on'):
        doulbucc=1
    else:
        doulbucc=0
    if(request.args.get('aphtes')=='on'):
        aphtes=1
    else:
        aphtes=0
    if(request.args.get('vomi')=='oui'):
        vomi=1
    else:
        vomi=0
    horodate=time.strftime('%d/%m/%y %H:%M',time.localtime())
    send_data={ 'id' : idPatient, 'nom': nomPatient, 'prenom' : pnomPatient, 'temperature' : temperature, 'fatigue' : nFatigue, 'sechbucc' : sechbucc, 'doulbucc' : doulbucc, 'aphtes' : aphtes, 'vomi' : vomi,'mg':MG, 'horodate' : horodate, 'age':age, 'taille':taille,'poids':poids}
    print(send_data)

    #Partie MQTT
    #Connection au broker
    try:
        #Mettre la ligne ci dessous en commentaire si test en local
        clientMosquitto.username_pw_set("patient1",password="p1agi")
	print(IPbroker)
	print(PORTbroker)
        clientMosquitto.connect(IPbroker,PORTbroker,60)
        clientMosquitto.loop_start()
        #Publier la trame
        clientMosquitto.publish(CHEMINtrame,str(send_data))
        #Dire à la page que la trame est envoyé
        clientMosquitto.loop_stop()
        return jsonify({"etat": "OK"})
    except:
        return jsonify({"etat": "Connection MQTT NOK"})

@app.route('/rMG/',methods=['GET'])
def rMG():
    	global age, taille, poids
    #try :
        bioimpedancemetre.open()
        #Paramètrage du canal 4 : Main et Cheville Droite
        bioimpedancemetre.write(b"p_c=4\r\n")
        out=bioimpedancemetre.readline()
        blocs=out.split(b' ')
        erreur=blocs[5]
        erreur=erreur[2:]
        erreur=erreur[:2]
	print(erreur)
        if(erreur!=b'00') :
            raise Exception

        #Mesure à 5 kHz
        bioimpedancemetre.write(b"m_f=50\r\n")
        out=bioimpedancemetre.readline()
        blocs=out.split(b' ')
        xbytes5=blocs[8]
        ybytes5=blocs[10]
        erreur=blocs[13]
        erreur=erreur[2:]
        erreur=erreur[:2]
	print(erreur)
        if(erreur!=b'00') :
            raise Exception
        xvalue5=float(xbytes5)
        yvalue5=abs(float(ybytes5))


        #Mesure à 50 kHz
        bioimpedancemetre.write(b"m_f=500\r\n")
        out=bioimpedancemetre.readline()
        blocs=out.split(b' ')
        xbytes50=blocs[8]
        ybytes50=blocs[10]
        erreur=blocs[13]
        erreur=erreur[2:]
        erreur=erreur[:2]
	print(erreur)
        if(erreur!=b'00') :
            raise Exception
        xvalue50=float(xbytes50)
        yvalue50=abs(float(ybytes50))

        #Mesure à 150 kHz
        bioimpedancemetre.write(b"m_f=1500\r\n")
        out=bioimpedancemetre.readline()
        blocs=out.split(b' ')
        xbytes150=blocs[8]
        ybytes150=blocs[10]
        erreur=blocs[13]
        erreur=erreur[2:]
        erreur=erreur[:2]
	print(erreur)
        if(erreur!=b'00') :
            raise Exception
        xvalue150=float(xbytes150)
        yvalue150=abs(float(ybytes150))

        #Mesure à 200 kHz
        bioimpedancemetre.write(b"m_f=2000\r\n")
        out=bioimpedancemetre.readline()
        blocs=out.split(b' ')
        xbytes200=blocs[8]
        ybytes200=blocs[10]
        erreur=blocs[13]
        erreur=erreur[2:]
        erreur=erreur[:2]
	print(erreur)
        if(erreur!=b'00') :
            raise Exception
        xvalue200=float(xbytes200)
        yvalue200=abs(float(ybytes200))

        #Mesure à 250 kHz
        bioimpedancemetre.write(b"m_f=2500\r\n")
        out=bioimpedancemetre.readline()
        blocs=out.split(b' ')
        xbytes250=blocs[8]
        ybytes250=blocs[10]
        erreur=blocs[13]
        erreur=erreur[2:]
        erreur=erreur[:2]
	print(erreur)
        if(erreur!=b'00') :
            raise Exception
        xvalue250=float(xbytes250)
        yvalue250=abs(float(ybytes250))

        #Mesure à 325 kHz
        bioimpedancemetre.write(b"m_f=3250\r\n")
        out=bioimpedancemetre.readline()
        blocs=out.split(b' ')
        xbytes325=blocs[8]
        ybytes325=blocs[10]
        erreur=blocs[13]
        erreur=erreur[2:]
        erreur=erreur[:2]
	print(erreur)
        if(erreur!=b'00') :
            raise Exception
        xvalue325=float(xbytes325)
        yvalue325=abs(float(ybytes325))

        #Arrêt de la liaison
        bioimpedancemetre.close()

        #######################################################

        # Calcul de la Masse Graisse


        #Déclarations des coefficients non calculés

        #Données Patient
        age=int(request.args.get('age'))
        taille=int(request.args.get('taille'))
        poids=int(request.args.get('poids'))

        #Coefficients Equation Homme : CD MG Db
        df=pd.read_excel('/home/pi/Downloads/Project/API-RASP/data.xls',sheetname='CEH')
        CeH=df['CD MG Db'][0]
        Cage=df['CD MG Db'][1]
        IMC=df['CD MG Db'][2]
        Xcp=[df['CD MG Db'][3],df['CD MG Db'][4],df['CD MG Db'][5],df['CD MG Db'][6],df['CD MG Db'][7],df['CD MG Db'][8]]
        CReRi=df['CD MG Db'][9]

        #Données Électriques
        DReac=[yvalue5,yvalue50,yvalue150,ybytes200,yvalue250,yvalue325]
        DRes=[xvalue5,xvalue50,xvalue150,xbytes200,xvalue250,xvalue325]

        #Données Numériques
        df=pd.read_excel('/home/pi/Downloads/Project/API-RASP/data.xls',sheetname='HSD')
        SDReac=[df['SD'][58],df['SD'][61],df['SD'][64],df['SD'][67],df['SD'][70],df['SD'][73]]
        SDRes=[df['SD'][57],df['SD'][60],df['SD'][63],df['SD'][66],df['SD'][69],df['SD'][72]]


        # Déclarations et Calculs des Équations

        #Coefficient Correcteur de Taille
        if(taille<158):
            CCT=(10000*poids)/(177*taille)
        else:
            CCT=(10000*poids)/(taille*taille)
        #Réactance à x kHz
        Reac=[]
        for i in range(len(DReac)):
            Reac.append(float(DReac[i])/SDReac[i])

        #Résistance à x kHz
        Res=[]
        for i in range(len(DRes)):
            Res.append(float(DRes[i])/SDRes[i])

        #Réactance équivalente à x kHz
        Req=[]
        for i in range(len(Res)):
            Req.append(abs(Reac[i])+(Res[i]*Res[i])/abs(Reac[i]))

        #Xo
        Xo=(Res[0]*Res[0]-Res[2]*Res[2]+Reac[0]*Reac[0]-Reac[2]*Reac[2])/(2*(Res[0]-Res[2]))

        #Résistance Extracellulaire
        Rext=abs(Xo+math.sqrt(Reac[0]*Reac[0]+(Res[0]-Xo)*(Res[0]-Xo)))

        #Résistance Infini
        Rinf=abs(Xo-math.sqrt(Reac[0]*Reac[0]+(Res[0]-Xo)*(Res[0]-Xo)))

        #Masse Grasse Homme Sain Debout
        MGHSD=abs((CeH+age*Cage+CCT*IMC+Req[0]*Xcp[0]+Req[1]*Xcp[1]+Req[2]*Xcp[2]+Req[3]*Xcp[3]+Req[4]*Xcp[4]+Req[5]*Xcp[5]+(Rext/Rinf)*CReRi)/1000)
        MHSDp100=(MGHSD/poids)*100
        global MG
        MG=round(MHSDp100,2)
        return jsonify({'etat':"OK",'MG':round(MG,2)})
    #except:
       # age=int(request.args.get('age'))
       # taille=int(request.args.get('taille'))
       # poids=int(request.args.get('poids'))
       # return jsonify({'etat':"NOK",'MG':None})

@app.route('/',methods=['GET'])
def Index():
    global ConfOK
    if ConfOK :
        return redirect(url_for('Questionnaire'))
    return render_template('index.html')


@app.route('/conf/',methods=['GET'])
def Conf():
    global nomPatient, pnomPatient, idPatient, IPbroker, PORTbroker, ConfOK
    nomPatient=str(request.args.get('nom'))
    pnomPatient=str(request.args.get('prenom'))
    idPatient=int(request.args.get('id'))
    IPbroker=request.args.get('adresseIP')
    PORTbroker=int(request.args.get('port'))
    ConfOK=Connect(request.args.get('ssid'),request.args.get('password'))
    if ConfOK:
        return jsonify({'etat' : 'Connection OK, veuillez rafraîchir la page'})
    else:
        return jsonify({'etat': 'Mauvaise Configuration'})


if __name__ == '__main__':
        app.run(host='192.168.200.1', port=PORTapi, debug=True)
